/*
   Silly Language S2.

   This parser does not "do" anything. It does not create a parse tree
   nor an abstract syntax tree. This parser is just a "recognizer" for
   the language. It will throw an exception if the string cannot be parsed.

   Notice that the two alternatives in the first production of this language have the same first token.

   S2 -> A '+' B | A '+' C
   A  -> 'a' '*' '(' S2 ')' | 'a'
   B  -> 'b' '*' '(' S2 ')' | 'b'
   C  -> 'c' '*' '(' S2 ')' | 'c'

   We can use EBNF to rewrite this grammar in a way that will work with a recursive descent parser.

   S2 -> A '+' ( B | C )
   A  -> 'a' [ '*' '(' S2 ')' ]
   B  -> 'b' [ '*' '(' S2 ')' ]
   C  -> 'c' [ '*' '(' S2 ')' ]


   Some example strings are:
   "a*(a+c)+b"
   "a*(a*(a+b)+b)+c*(a+b*(a+c))"
*/

public class S2Parse
{
   public static void recognizeS2(String stringInS2) throws ParseException
   {
      Tokenizer tokens = new Tokenizer(stringInS2);

      getS2(tokens);
   }//recognizeS2()


   public static void getS2(Tokenizer tokens) throws ParseException
   {
      getA(tokens);
      String token = tokens.nextToken();
      if ( !token.equals("+") )
         throw new ParseException("Can't parse " + token);
      token = tokens.peekToken();
      if ( token.equals("b") )
      {
         getB(tokens);
      }
      else if ( token.equals("c") )
      {
         getC(tokens);
      }
      else
         throw new ParseException("Can't parse.");
   }//getS2()


   public static void getA(Tokenizer tokens) throws ParseException
   {
      String token = tokens.nextToken();
      if ( !token.equals("a") )
         throw new ParseException("Can't parse " + token);
      if ( tokens.hasToken() )
      {
         token = tokens.peekToken();
         if ( token.equals("*") )
         {
            tokens.nextToken();  // consume the *
            token = tokens.nextToken();
            if ( !token.equals("(") )
               throw new ParseException("Can't parse " + token);
            getS2(tokens);
            token = tokens.nextToken();
            if ( !token.equals(")") )
               throw new ParseException("Can't parse " + token);
         }
      }
   }//getA()


   public static void getB(Tokenizer tokens) throws ParseException
   {
      String token = tokens.nextToken();
      if ( !token.equals("b") )
         throw new ParseException("Can't parse " + token);
      if ( tokens.hasToken() )
      {
         token = tokens.peekToken();
         if ( token.equals("*") )
         {
            tokens.nextToken();  // consume the *
            token = tokens.nextToken();
            if ( !token.equals("(") )
               throw new ParseException("Can't parse " + token);
            getS2(tokens);
            token = tokens.nextToken();
            if ( !token.equals(")") )
               throw new ParseException("Can't parse " + token);
         }
      }
   }//getB()


   public static void getC(Tokenizer tokens) throws ParseException
   {
      String token = tokens.nextToken();
      if ( !token.equals("c") )
         throw new ParseException("Can't parse " + token);
      if ( tokens.hasToken() )
      {
         token = tokens.peekToken();
         if ( token.equals("*") )
         {
            tokens.nextToken();  // consume the *
            token = tokens.nextToken();
            if ( !token.equals("(") )
               throw new ParseException("Can't parse " + token);
            getS2(tokens);
            token = tokens.nextToken();
            if ( !token.equals(")") )
               throw new ParseException("Can't parse " + token);
         }
      }
   }//getC()





   public static void main(String[] args) throws ParseException
   {
      String[] stringsInS2 = { "a * ( a + c ) + b",
                               "a * ( a * ( a + b ) + b ) + c * ( a + b * ( a + c ) )",
                               "b * ( a * ( b + a ) + b ) + a * ( b + b * ( b + a ) )" };

      for (int i = 0; i < stringsInS2.length; i++)
      {
         System.out.println( "Parsing " + stringsInS2[i] );
         recognizeS2( stringsInS2[i] );
         System.out.println( " ... Parsed" );
      }
   }//main()

}//S2Parse
