/*
   Silly Language S4.

   This parser does not "do" anything. It does not create a parse tree
   nor an abstract syntax tree. This parser is just a "recognizer" for
   the language. It will throw an exception if the string cannot be parsed.

   Notice that the first production is right recursive..

   S4 -> A '+' S4 | B '+' S4 | A | B
   A  -> 'a' '*' '(' S4 ')' | 'a'
   B  -> 'b' '*' '(' S4 ')' | 'b'

   We can use EBNF to rewrite this grammar.

   S4 -> ( A | B ) [ '+' S4 ]
   A  -> 'a' [ '*' '(' S4 ')' ]
   B  -> 'b' [ '*' '(' S4 ')' ]

   Some example strings are:
   "a*(a+b)+a*(a+b)"
   "b*( a*(b+b)+b ) + a*(a+b*(a+a))"
*/

public class S4Parse
{
   public static void recognizeS4(String stringInS4) throws ParseException
   {
      Tokenizer tokens = new Tokenizer(stringInS4);

      getS4(tokens);
   }//parecognizeS4()


   public static void getS4(Tokenizer tokens) throws ParseException
   {
      String token = tokens.peekToken();  // we "peek" ahead to help us
      if ( token.equals("a") )            // decide which case we're in
      {
         getA(tokens);
      }
      else if ( token.equals("b") )
      {
         getB(tokens);
      }
      else
         throw new ParseException("Can't parse.");

      if ( tokens.hasToken() )
      {
         token = tokens.peekToken();
         if ( token.equals("+") )
         {
            tokens.nextToken();  // consume the +
            getS4(tokens); // recursive call S4()
         }
      }
   }//getS4()


   public static void getA(Tokenizer tokens) throws ParseException
   {
      String token = tokens.nextToken();
      if ( !token.equals("a") )
         throw new ParseException("Can't parse " + token);
      if ( tokens.hasToken() )
      {
         token = tokens.peekToken();
         if ( token.equals("*") )
         {
            tokens.nextToken();  // consume the *
            token = tokens.nextToken();
            if ( !token.equals("(") )
               throw new ParseException("Can't parse " + token);
            getS4(tokens);
            token = tokens.nextToken();
            if ( !token.equals(")") )
               throw new ParseException("Can't parse " + token);
         }
      }
   }//getA()


   public static void getB(Tokenizer tokens) throws ParseException
   {
      String token = tokens.nextToken();
      if ( !token.equals("b") )
         throw new ParseException("Can't parse " + token);
      if ( tokens.hasToken() )
      {
         token = tokens.peekToken();
         if ( token.equals("*") )
         {
            tokens.nextToken();  // consume the *
            token = tokens.nextToken();
            if ( !token.equals("(") )
               throw new ParseException("Can't parse " + token);
            getS4(tokens);
            token = tokens.nextToken();
            if ( !token.equals(")") )
               throw new ParseException("Can't parse " + token);
         }
      }
   }//getB()





   public static void main(String[] args) throws ParseException
   {
      String[] stringsInS4 = { "a * ( a + b ) + a * ( a + b ) + b",
                               "b * ( a * ( b + b ) + b ) + a * ( a + b * ( a + a ) )",
                               "b * ( a * ( b + a ) + b ) + a * ( b + b * ( b + a ) )" };

      for (int i = 0; i < stringsInS4.length; i++)
      {
         System.out.println( "Parsing " + stringsInS4[i] );
         recognizeS4( stringsInS4[i] );
         System.out.println( " ... Parsed" );
      }
   }//main()

}//S4Parse
