/**
   This program tests the Parse2AST parser.

   When this program runs, it creates dot and
   png files for each of the abstract syntax trees.

   You can download the installer for dot.exe from this URL.
     https://graphviz.gitlab.io/_pages/Download/windows/graphviz-2.38.zip
*/

public class TestInfix3
{
   public static void main(String[] args)
   {
      String infix[] =
            // parenthesized infix notation
          { "sqrt 25",
            "2 + 3 * 4",
            "( 2 + 3 ) * 4",
            "2 - 3 + 4",
            "2 - ( 3 + 4 )",
            "4 / 2 / 2",
            "4 / ( 2 / 2 )",
            "-2 ^ 3",
            "( -2 ) ^ 3",
            "( - 2 ) ^ 3",

            "3 * sqrt 25 - ( - 6 + 5 ) * - 1",
            "- sqrt ( 6 ^ 2 ) + ( ( 4 ) * ( 2 ) )",

            "( 2 + 3 * 4 )",
            "( 2 + ( 3 * 4 ) )",
            "( ( 2 ) + ( 3 ) * ( 4 ) )",
            "( ( 2 ) + ( ( 3 ) * ( 4 ) ) )",

            "3 ^ 4  7 * 8",           // syntax error
            "3 ^ 4 * 7 +",            // syntax error (at +)
            "3 ^ 4 + ( 7 @ 8 )",      // syntax error (at @)
            "3 ^ 4 + 7 * x"           // syntax error (at x)
          };


      for (int i = 0; i < infix.length; i++)
      {
         try  // build the abstract syntax tree that represents the expression
         {
            Tree expTree = Parse2AST.parse( infix[i] );
            System.out.println( expTree + "\n");

            try  // Create dot and png files from the AST.
            {
               // Create the (empty) dot file.
               String baseName = String.format("Infix_Expression_%02d_AST", i);
               java.io.PrintWriter out = new java.io.PrintWriter(
                                         new java.io.File(baseName + ".dot") );
               // Put dot commands into the dot file
               out.println("//  " + expTree + "\n");
               out.println( Tree2dot.tree2dot(expTree) + "\n" );
               out.close();
               // Create a command line for running dot.exe.
               String[] cmd = {"C:\\graphviz-2.38\\release\\bin\\dot.exe",
                               "-Tpng",
                               baseName + ".dot",
                               "-o",
                               baseName + ".png"};
               // Execute the command line.
               java.lang.Runtime.getRuntime().exec(cmd);
            }
            catch (Exception e)
            {
               System.out.println(e);
            }
         }
         catch (ParseException e)
         {
            System.out.println(e + "\n");
          //e.printStackTrace();
         }
      }
   }//main()

}//TestInfix3
